/**************************		mov.h	************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <termio.h>
#include <unistd.h>

typedef enum boolean {F, T} Bool;

typedef struct Data {
	int stime;
	int stemp;
	int ctime;
	int ctemp;
	//
	int i_cmode;
};

typedef struct State {
	Bool DoorState;
	Bool Time_TempButtonState;
	Bool ModeButtonState;
	Bool _10sec_10CButtonState;
	Bool _30sec_20CButtonState;
	Bool S_CButtonState;
};

//////////////////////////////////////////////////////////////////////
/*
	public function
*/
int getch(void)
{
        int ch;
 
        struct termios buf;
        struct termios save;
 
        tcgetattr(0, &save);
        buf = save;
 
        buf.c_lflag &= ~(ICANON|ECHO);
        buf.c_cc[VMIN] = 1;
        buf.c_cc[VTIME] = 0;
 
        tcsetattr(0, TCSAFLUSH, &buf);
 
        ch = getchar();
 
        tcsetattr(0, TCSAFLUSH, &save);
 
        return ch;
}

int kbhit(void){  
	struct termios oldt, newt;

	int ch;  
	int oldf;   

	tcgetattr(STDIN_FILENO, &oldt);  
	newt = oldt;  
	newt.c_lflag &= ~(ICANON | ECHO);  
	tcsetattr(STDIN_FILENO, TCSANOW, &newt); 
	oldf = fcntl(STDIN_FILENO, F_GETFL, 0); 
	fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK); 
	ch = getchar();  
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt); 
	fcntl(STDIN_FILENO, F_SETFL, oldf); 

	if(ch != EOF)  {   
		ungetc(ch, stdin);    
		return 1;
	}   

	return 0;
}

/*
	end public function
*/

/////////////////////////////////////////////////////////////////////
/*
	interface
*/
//Input
void DoorSenSorInterface(Bool DoorSensorInput);											// 1.1
void Time_TempInterface(Bool Time_TempInput);												// 1.2
void ModeButtonInterface(Bool ModeButtonInput);											// 1.3
void _10sec_10CButtonInterface(Bool _10sec_10C_increButtonInput);						// 1.4
void _30sec_20CButtonInterface(Bool _30sec_20C_increButtonInput);						// 1.5
void S_CButtonInterface(Bool S_CButtonInput);												// 1.6
//Output
void Time_TempDisplayInterface(Bool TimeDisplayCommand, Bool TempDisplayCommand);		// 2.2
void ModeDisplayInterface(Bool ModeDisplayCommand);										// 2.3
void LighterInterface(Bool BacklightCommand);												// 2.4
void SpeakerInterface(Bool SpeakCommand);													// 2.5
/*
	end interface
*/

//////////////////////////////////////////////////////////////////////
/*
	buttoncontroller
*/
void buttoncontroller (struct State *s);													// 2.1.1

void _10secIncrease (Bool trigger);															// 2.1.2
void _10CIncrease (Bool trigger);															// 2.1.3
void _30secIncrease (Bool trigger);															// 2.1.4
void _20CIncrease (Bool trigger);															// 2.1.5

void ManualMode (Bool trigger);																// 2.1.6
void DduckMode (Bool trigger);																// 2.1.7
void JukMode (Bool trigger);																	// 2.1.8
void BobMode (Bool trigger);																	// 2.1.9
void Kuk_JjigaeMode (Bool trigger);															// 2.1.10
void PizzaMode (Bool trigger);																// 2.1.11
/*
	end buttoncontroller
*/

/////////////////////////////////////////////////////////////////////
/*
	time_tempcontroller
*/
void Time_TempController(struct Data *d, struct State *s);								// 2.1.12

void Light(Bool enable, Bool disable);														// 2.1.13
void Make_Sound(Bool trigger);																// 2.1.14
void TempIncreasing(Bool trigger);															// 2.1.15
void TimeDecreasing(Bool trigger);															// 2.1.16
/*
	end time_tempcontroller
*/

////////////////////////////////////////////////////////////////////
/*
	displaycontroller
*/
void DisplayController(struct Data *d, struct State *s);									// 2.1.17

void TimePrint(Bool enable, Bool disable);													// 2.1.18
void TempPrint(Bool enable, Bool disable);													// 2.1.19
void ModePrint(Bool enable, Bool disable);													// 2.1.20
/*
	end displaycontroller
*/

/**************************		end		mov.h	************************/












/**************************		main.c		************************/

extern int ABLE = 1;	// odd is Time, even is Temp
extern int DOOR = 1;
extern int SC = 0;
extern struct Data d={0, 20, -1, -1, 0, /*"Manual"*/};
extern struct State s={F, F, F, F, F, F};

extern Bool enable = F;
extern Bool disable = F;
extern Bool trigger = F;

/* !!modified code - added part*/
void DataClear()
{
	d.stime = 0;
	d.stemp = 20;
	d.ctime = -1;
	d.ctemp = -1;
	d.i_cmode = 0;
	
}
void StateClear()
{
	s.DoorState = F;
	s.Time_TempButtonState = F;
	s.ModeButtonState = F;
	s._10sec_10CButtonState = F;
	s._30sec_20CButtonState = F;
	s.S_CButtonState = F;
}
/* modified code!! - added part*/

int main_controller() {

	char userInput, a;

	Bool DoorSensorInput = F;
	Bool Time_TempInput = F;
	Bool ModeButtonInput = F;
	Bool _10sec_10C_increButtonInput = F;
	Bool _30sec_20C_increButtonInput = F;
	Bool S_CButtonInput = F;
	
	//printf("0:00\tMode : Manual\n");
	
	do {
		userInput = getch();

		SC = 0;

		Time_TempInput = F;
		ModeButtonInput = F;
		_10sec_10C_increButtonInput = F;
		_30sec_20C_increButtonInput = F;
		S_CButtonInput = F;
		DoorSensorInput = F;

		switch(userInput){
			case 'a':
				_10sec_10C_increButtonInput = T;
				_10sec_10CButtonInterface(_10sec_10C_increButtonInput);				// 1.4
				buttoncontroller(&s);													// 2.1.1
				Time_TempController(&d, &s);												// 2.1.12
				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'b':
				_30sec_20C_increButtonInput = T;										
				_30sec_20CButtonInterface(_30sec_20C_increButtonInput);				// 1.5
				buttoncontroller(&s);													// 2.1.1
				Time_TempController(&d, &s);												// 2.1.12
				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'c':
				/* !!modified code - added part */
				if(d.i_cmode != 0) break;
				/* modified code!! - added part*/
				Time_TempInput = T;
				Time_TempInterface(Time_TempInput);										// 1.2
				buttoncontroller(&s);													// 2.1.1
				Time_TempController(&d, &s);												// 2.1.12
				
				/* !!modified code  - added part*/
				DataClear();
				StateClear();
				/* modified code!!  - added part*/
				
				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'd':
				ModeButtonInput = T;
				ModeButtonInterface(ModeButtonInput);									// 1.3
				buttoncontroller(&s);													// 2.1.1
				Time_TempController(&d, &s);												// 2.1.12
				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'e':
				/* !!modified code  - added part*/
				if((d.stime == 0) && (d.stemp == 20)) break;
				if((DOOR%2) == 0) break;
				/* modified code!!  - added part*/
				d.ctime = d.stime;
				d.ctemp = 20;
				S_CButtonInput = T;
				S_CButtonInterface(S_CButtonInput);										// 1.6
				buttoncontroller(&s);													// 2.1.1
				while ( SC != 1 ) {
					if(kbhit())	{
						a = getch();
						/* !!modified code  - modified part*/
						if(a =='e')	{
							s.S_CButtonState = F;
							DataClear(); // 기존의 초기화 코드에서 함수로 바뀌었다.
							
							Light(F, T);													// 2.1.13
							DisplayController(&d, &s);									// 2.1.17
							
							break;
						}
						/* modified code!!  - modified part*/
					}

					Time_TempController(&d, &s);											// 2.1.12
					DisplayController(&d, &s);											// 2.1.17
				}
				s.S_CButtonState = F;

				/* !!modified code  - modified part*/
				DataClear(); // 기존의 초기화 코드에서 함수로 바뀌었다.
				/* modified code!!  - modified part*/

				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'f':
				DoorSensorInput = T;
				DoorSenSorInterface(DoorSensorInput);									// 1.1
				buttoncontroller(&s);													// 2.1.1
				Time_TempController(&d, &s);												// 2.1.12
				DisplayController(&d, &s);												// 2.1.17
				break;
			case 'x':
				printf("exit program\n");
				break;
			default:
				printf("please collect menu order\n");
				break;
		}
	} while ('x' != userInput);
	

	return 0;
}
/**************************		end		main.c	************************/














/**************************		interface.c		************************/
/*
Input
*/
void DoorSenSorInterface (Bool DoorSensorInput){										// 1.1
	s.DoorState = DoorSensorInput;
}

void Time_TempInterface (Bool Time_TempInput){										// 1.2
	s.Time_TempButtonState = Time_TempInput;
}

void ModeButtonInterface (Bool ModeButtonInput){										// 1.3
	s.ModeButtonState = ModeButtonInput;
}

void _10sec_10CButtonInterface (Bool _10sec_10C_increButtonInput){					// 1.4
	s._10sec_10CButtonState = _10sec_10C_increButtonInput;
}

void _30sec_20CButtonInterface (Bool _30sec_20C_increButtonInput){					// 1.5
	s._30sec_20CButtonState = _30sec_20C_increButtonInput;
}

void S_CButtonInterface (Bool S_CButtonInput){										// 1.6
	s.S_CButtonState = S_CButtonInput;
} 

///////////////////////////////////////////////////////////////////



/*
Output
*/
void Time_TempDisplayInterface(Bool TimeDisplayCommand, Bool TempDisplayCommand) {// 2.2
	//system("clear");
	if ( (T == TimeDisplayCommand) && (F == TempDisplayCommand)) {
		if ( d.ctime == -1 ) {
			//printf("%d:%02d\t", (d.stime)/60, (d.stime)%60);
		}
		else {
			//printf("%d:%02d\t", (d.ctime)/60, (d.ctime)%60);
		}
	}
	else {
		if ( d.ctemp == -1 ) {
			//printf("%dC\t", d.stemp);
		}
		else {
			//printf("%dC\t", d.ctemp);
		}
	}
}
void ModeDisplayInterface(Bool ModeDisplayCommand) {									// 2.3

	/*if (ModeDisplayCommand == T) {
		//printf("Mode : ");

		switch (d.i_cmode) {
		case 0:
			//printf("Manual\n");
			break;
		case 1:
			//printf("Dduck\n");
			break;
		case 2:
			//printf("Juk\n");
			break;
		case 3:
			//printf("Bob\n");
			break;
		case 4:
			//printf("Kuk_Jjigae\n");
			break;
		case 5:
			//printf("Pizza\n");
			break;
		default :
			printf("Error, in switch of ModeDisplayInterface.\n");
		}
	}*/
	//printf("\n");
}
void SpeakerInterface (Bool SpeakCommand) {											// 2.4
	int i;
	char a;

	if (T == SpeakCommand) {
		for(i=0; i<3; i++) {
			/* !!modified code */
			printf("\a");
			//sleep(1);
			if(kbhit())	{
				a = getch();
				if(a =='e')	{
				s.S_CButtonState = F;
				DataClear();
						
				Light(F, T);
				DisplayController(&d, &s);
					
				break;
				}
			}
			/* modified code!! */
		}
		SC=1;
	}
}
void LighterInterface(Bool BacklightCommand) {										// 2.5
	if ( (T == BacklightCommand) ) {
		printf("%c[1;33m", 27);
	}
	else {
		printf("%c[0m", 27);
	}
}
/**************************		end		interface.c		************************/

/**************************		buttoncontroller.c		************************/
void buttoncontroller (struct State *s){												// 2.1.1

	static int state = 1;			// odd : Time, even : Temp
	static int mode = 0;				// Mode
	trigger = F;

	if(s->Time_TempButtonState == T){
		//state++;
		ABLE++; // sum
		s->Time_TempButtonState = F;

		DataClear();
	}
	///
	if( ( s->ModeButtonState == T ) && ( (ABLE % 2) != 0 ) ){
				d.i_cmode++;
				d.i_cmode %= 6;
				/* !!modified code  - added part*/
				if ( d.i_cmode == 0 ) {
					DataClear();
					StateClear();
				}
				/* modified code!!  - added part*/
				s->ModeButtonState = F;
	}
	///
	if (d.i_cmode == 0) {
		trigger = T;
		ManualMode(trigger); // 2.1.6
		
		if( ( s->_10sec_10CButtonState == T ) && ( (ABLE % 2) != 0 ) ) {
			s->_10sec_10CButtonState = F;
			trigger = T;		
			_10secIncrease(trigger); // 2.1.2
		}
		if( ( s->_30sec_20CButtonState == T ) && ( (ABLE % 2) != 0 ) ){
			s->_30sec_20CButtonState = F;
			trigger = T;		
			_30secIncrease(trigger);	// 2.1.4
		}
		////
		if( ( s->_10sec_10CButtonState == T ) && ( (ABLE % 2) == 0 ) ) {
			s->_10sec_10CButtonState = F;
			trigger = T;		
			_10CIncrease(trigger); // 2.1.3
		}
		if( ( s->_30sec_20CButtonState == T ) && ( (ABLE % 2) == 0 ) ){
			s->_30sec_20CButtonState = F;
			trigger = T;		
			_20CIncrease(trigger); // 2.1.5
		}

	}
	else {
			switch(d.i_cmode){
				case 1: // Dduck Mode
					trigger = T;
					DduckMode(trigger); // 2.1.7
					break;
				case 2: // Juk Mode
					trigger = T;
					JukMode(trigger);	 // 2.1.8
					break;
				case 3: // Bob Mode
					trigger = T;
					BobMode(trigger); // 2.1.9
					break;
				case 4: // Kuk_Jjigae Mode
					trigger = T;
					Kuk_JjigaeMode(trigger); // 2.1.10
					break;
				case 5: // Pizza Mode
					trigger = T;
					PizzaMode(trigger); // 2.1.11
					break;
				default :
					printf("Error, in switch of burroncontroller.\n");
					break;
			} // switch
	}
}

void _10secIncrease (Bool trigger) {													// 2.1.2
	if(T == trigger) {
		d.stime += 10;
		d.stime %= 600;
	}
}
void _10CIncrease (Bool trigger) {														// 2.1.3
	if(T == trigger) {
		d.stemp += 10;
		if(d.stemp > 90) {
			d.stemp = (d.stemp % 70);
		}
	}
}
void _30secIncrease (Bool trigger) {													// 2.1.4
	if(T == trigger) {
		d.stime += 30;
		d.stime %= 600;
	}
}
void _20CIncrease (Bool trigger) {														// 2.1.5
	if(T == trigger)		
		d.stemp += 20;
		if(d.stemp > 90) {
			d.stemp = (d.stemp % 70);
		}
}

void ManualMode (Bool trigger) {														// 2.1.6
	if(T == trigger) {			
		d.i_cmode = 0;
	}
}
void DduckMode (Bool trigger) {															// 2.1.7
	if(T == trigger) {			
		d.stime = 60;
		d.i_cmode = 1;
	}
}
void JukMode (Bool trigger) {															// 2.1.8
	if(T == trigger) {			
		d.stime = 90;
		d.i_cmode = 2;
	}
}
void BobMode (Bool trigger) {															// 2.1.9
	if(T == trigger) {			
		d.stime = 120;
		d.i_cmode = 3;
	}
}
void Kuk_JjigaeMode (Bool trigger) {													// 2.1.10
	if(T == trigger) {			
		d.stime = 300;
		d.i_cmode = 4;
	}
}
void PizzaMode (Bool trigger) {															// 2.1.11
	if(T == trigger) {			
		d.stime = 120;
		d.i_cmode = 5;
	}
}
/**************************		end		buttoncontroller.c		************************/

/**************************		time_tempcontroller.c		************************/
void Time_TempController(struct Data *d, struct State *s){							// 2.1.12

	trigger = F;
	enable = F;
	disable = F;

	if ( s->DoorState == T ) {
		s->DoorState = F;
		DOOR++;
		if ( (DOOR % 2) == 0 ) {
			enable = T;
		}
		else {
			disable = T;
		}
		Light(enable, disable); // 2.1.13
	}
	else {
	
		if ( (s->S_CButtonState == T) && ((DOOR%2) != 0) ) {
			if( (ABLE % 2) != 0 ){ 			// When ModeState is Time.
				if (d->ctime == 0 ) {
					trigger = T;
					disable = T;
					Light(enable, disable); // 2.1.13
					Make_Sound(trigger); // 2.1.14
				}
				else {
					trigger = T;
					enable = T;
					Light(enable, disable); // 2.1.13
					TimeDecreasing(trigger); // 2.1.16
				}
			}
			else if( (ABLE % 2) == 0 ){				// When ModeState is Temp.
				if (d->ctemp >= d->stemp ) {
					trigger = T;
					disable = T;
					Light(enable, disable); // 2.1.13
					Make_Sound(trigger); // 2.1.14
				}
				else {
					trigger = T;
					enable = T;
					Light(enable, disable); // 2.1.13
					TempIncreasing(trigger); // 2.1.15
				}
			}
		} // if
		else {
			s->S_CButtonState = F;
		}
	} // else
}

void TimeDecreasing (Bool trigger){														// 2.1.16
	//DisplayController(&d, &s); // 2.1.17
	//sleep(1);
	d.ctime = d.ctime - 1;
}

void TempIncreasing (Bool trigger){														// 2.1.15
	//DisplayController(&d, &s); // 2.1.17
	//sleep(3);
	d.ctemp = d.ctemp + 10;
}

void Make_Sound (Bool trigger){															// 2.1.14
	Bool SpeakCommand = F;	
	if (T == trigger) {
		SpeakCommand = T;	
		SpeakerInterface(SpeakCommand); // 2.4
	}
	else {
		SpeakCommand = F;
		SpeakerInterface(SpeakCommand); // 2.4
	}
}
void Light(Bool enable, Bool disable) {												// 2.1.13
	Bool BacklightCommand = F;
	if ( (T == enable) && (F == disable) ) {
		BacklightCommand = T;
		LighterInterface(BacklightCommand); // 2.5
	}
	else {
		LighterInterface(BacklightCommand); // 2.5
	}
}
/**************************		end		time_tempcontroller.c		************************/

/**************************		displaycontroller.c		************************/
void DisplayController(struct Data *d, struct State *s) {							// 2.1.17
	enable = F;
	disable = F;

	if ( (ABLE % 2) != 0 ) {
		enable = T;
		TimePrint(enable, disable); // 2.1.18
		ModePrint(enable, disable); // 2.1.20
	}
	else {
		enable = T;
		TempPrint(enable, disable); // 2.1.19
		ModePrint(enable, disable); // 2.1.20
	}
}


void TimePrint(Bool enable, Bool disable) {											// 2.1.18
	Bool TimeDisplayCommand = F;
	Bool TempDisplayCommand = F;

	if ( (enable == T) && (disable == F) ) {
		TimeDisplayCommand = T;
		Time_TempDisplayInterface(TimeDisplayCommand, TempDisplayCommand); // 2.2
	}
}
void TempPrint(Bool enable, Bool disable) {											// 2.1.19
	Bool TimeDisplayCommand = F;
	Bool TempDisplayCommand = F;

	if ( (enable == T) && (disable == F) ) {
		TempDisplayCommand = T;
		Time_TempDisplayInterface(TimeDisplayCommand, TempDisplayCommand); // 2.2
	}
}
void ModePrint(Bool enable, Bool disable) {											// 2.1.20
	Bool ModeDisplayCommand = F;
	
	if ( (enable == T) && (disable == F) ) {
		ModeDisplayCommand = T;
		ModeDisplayInterface(ModeDisplayCommand); // 2.3
	}
}

/**************************		end		displaycontroller.c		************************/
